#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "diz.h"


// creo un nuovo dizionario (o hash-table di dimensioni massime fissate)
struct Dizionario * nuovoDizionario( void )
{
    int i;

    /* alloca la memoria necessaria per contenere l'handler del dizionario */
    struct Dizionario * result = malloc( sizeof(struct Dizionario) );

    /* controlla che l'allocazione sia anadata a buon fine... */
    if(result==NULL) return NULL;

    /* inizializza i puntatori dell'handler a NULL */
    for(i=0;i<26;i++) result->index[i]=NULL;

    /* ritorna il puntatore all'handler del dizionario */
    return result;
}

// disalloco il dizionario (o hash-table)
int freeDizionario(struct Dizionario * dizionario)
{
    if(!dizionario) return NULL_POINTER;

    int i;
    struct DizNodo * attuale;
    struct DizNodo * pros;

    for(i=0;i<=25;i++)
    {
        attuale=dizionario->index[i];
        while(attuale)
        {
            pros=attuale->next;
            free(attuale->key);
            free(attuale->value);
            free(attuale);
            attuale=pros;
        }
    }

    free(dizionario);
    return RESULT_OK;
}

// inserisco un nuovo valore in corrispondenza della chiave
int inserisciValore(char * chiave, char * valore, struct Dizionario * diz)
{
    /* tests preliminari */
    if(!chiave || !diz) return NULL_POINTER;
    if(!*chiave) return EMPTY_STRING;

    /* controlla se la chiave non e' gia' presente nel dizionario */
    if(cercaValore(chiave, diz, &valore)==RESULT_OK) return DOUBLE_KEY_FOUND;

    /* calcolo l'indice della hash-table dove inserire la chiave */
    int i = tolower(*chiave) - 'a';

    /* creo il nuovo nodo */
    struct DizNodo * nuovo;
    nuovo = (struct DizNodo *) malloc( ( sizeof(struct DizNodo) ) );
    if(!nuovo) return MEM_ALLOC_ERROR;

    /* inserisco la chiave e il valore */
    nuovo->key = strdup(chiave); /* inserisco la chiave */
    nuovo->value = strdup(valore); /* inserisco il valore double */
    nuovo->next=diz->index[i]; /* setto il puntatore del prossimo nodo */
    diz->index[i]=nuovo; /* inserisco l'elemento in testa alla lista */

    return RESULT_OK;
}

// cerco un valore nel dizionario (o hashtable), data la chiave
int cercaValore(char * chiave, struct Dizionario * diz, char ** valore)
{
    if(!chiave || !diz || !valore) return NULL_POINTER;
    if(!*chiave) return EMPTY_STRING;

    /* calcolo l'indice della hash-table dove cercare la chiave */
    int i = tolower(*chiave) - 'a';

    struct DizNodo * attuale;
    attuale=diz->index[i];
    while(attuale)  /* scorre tutta la lista finche' non trova la chiave (o finche' non finisce la lista) */
    {
        if(strcmp(chiave, attuale->key)==0)
        {
            *valore=attuale->value;
            return RESULT_OK;
        }
        attuale=attuale->next;
    }

    /* else */
    return KEY_NOT_FOUND;
}
