/* Generic queue functions
with handler for fast insert
(use this instead of list.h if you have to manage large queues)
by Mantox, 2005

 Description:
The queue is a FIFO data structure.
That means that the First element In (when you do an enqueue)
is the First that exit Out (when you do a dequeue)

To create a queue with this library, first declare a queue like this:

    queue q=createqueue();

Then you can insert and remove elements using the specific
"enqueue" and "dequeue" functions (see below for prototypes).

    *** Function prototypes ***

queue createqueue( void );                     <-- used to create a new queue (handler)

void enqueue(queue q, quque_type data)         <-- used to put the element 'data' in the queue

queue_type dequeue(queue q)                    <-- used to read and remove the first element of the queue.

void printqueue(queue q)                       <-- prints the queue

void freequeue(queue q)                        <-- frees all the queue's nodes

*/

#include <stdio.h>
#include <stdlib.h>
#include "ush.h"

#if !defined  QUEUE_H && !defined LIST_H
#define QUEUE_H

/* Sets here the kind of data you are goint to store in the queue */
typedef cmdPtr queue_type;

struct queuehandler
{
    struct queuenode * head; /* a pointer to the head of the queue */
    struct queuenode * tail; /* a pointer to the head of the queue */
    int size; /* this field stores the number of nodes in the queue */
};

struct queuenode
{
    struct queuenode * next; /* a pointer to the next node */
    queue_type data;
};

typedef struct queuehandler * queue;

queue createqueue( void );

void enqueue(queue q, queue_type data);

queue_type dequeue(queue q);

void printqueue(queue q);

void freequeue(queue q);

int isempty(queue q);


#endif /* QUEUE_H */
