#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include "ush.h"
#include "queue.h"
#include "mbool.h"
#include "ush-var.h"
#include "ush-args.h"
#include "diz.h"

extern char *command;
extern bool script_mode;
extern hash aliases;
extern queue parsed_commands;

static const char *help =
"ush - micro shell                                           \n\n"
"syntax:                                                     \n"
//" ush -c <command>         - run a specified command and quit\n"
" ush                         - start ush in interactive mode\n"
" ush <script file> [script arguments]    - run a script file\n"
" ush -h                             - print this help screen\n"
"";

// private module function prototypes
#define isfile( filename ) ( ! access( filename, F_OK ) )
static void initialize_ush_script_mode( int argc, char *argv[] );
static void run_startup_script( void );

void check_args( int argc, char *argv[] )
{
    if ( argc > 1 )
    {
        if ( isfile(argv[1]) )
        {
            if ( is_script_file(argv[1]) )
            {
                initialize_ush_script_mode( argc, argv );
                script_mode = 1;
                return;
            }
            else
            {
                fprintf( stderr, "ush: %s: invalid script file. Read the documentation about scripting\n", argv[1] );
                exit(127);
            }
        }
        /*
        else if ( strcmp( argv[1], "-c" ) == 0 )
        {
            int i=2;
            while ( i < argc )
            {
                strcat( command, argv[i] );
                strcat( command, " " );
                i++;
            }
            strcat( command, "; exit" ); // assure exiting after executing the command
            return;
        }
        */
        else if ( strcmp( argv[1], "-h" ) == 0 )
        {
            puts( help );
            exit(0);
        }
        else
        {
            fprintf( stderr, "ush: %s: invalid argument\n", argv[1] );
            exit(127);
        }
    }
    else
        run_startup_script();

    return;
}

bool is_script_file( char *filename )
{
    if(!*filename) return 0;

    int fd = open( filename, O_RDONLY );
    if ( fd < 0 ) return 0;

    char header[13] = { 0 };
    read( fd, header, 11 );
    close ( fd );

    //to be considered "valid" a script file should have the right header
    return !strncmp( header, "#! /bin/ush", 11);
}


static void initialize_ush_script_mode( int argc, char *argv[] )
{
    // restore default SIGINT and SIGQUIT behaviour
    // (let user terminate the script pressing CTRL-C)
    signal( SIGINT, SIG_DFL );
    signal( SIGQUIT, SIG_DFL );

    // redirect stdin from file specified by agv[1]
    int t = open ( argv[1], O_RDONLY );
    if ( t<0 )
    {
        fprintf( stderr, "ush: %s: %s\n", argv[1], strerror(errno) );
        exit(127);
    }
    else
        dup2( t, STDIN_FILENO );

    /* unnecessary?
    // read script arguments and write $0-9 variables
    int i;
    for ( i = 2; (i < argc) && (i < 10); i++ )
        write_variable( itoa(i-2), argv[i-1] );

    // "$n" = number of arguments passed to the script
    write_variable( "n", itoa(argc-2) );

    // TODO: "$*" all arguments
    */

    return;
}

static void run_startup_script( void )
{
    char *startup_file = calloc( sizeof(char), 300 );
    sprintf( startup_file, "%s/%s", read_variable("HOME"), ".ushrc" );

    if( isfile( startup_file ) )
    {
        // creates a parsed command istance and execute it
        cmdPtr t = create_and_init_parsed_command();
        t->arguments = calloc( sizeof(char*), 2 );
        t->arguments[0] = strdup("ush");
        t->arguments[1] = startup_file;
        t->standard_input = -1;
        t->standard_output = -1;
        t->standard_error = -1;
        t->background = 0;
        enqueue( parsed_commands, t );
        execute_parsed_commands();
    }
}
