#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "ush-var.h"
#include "mstring.h"
#include "ush-icmd.h"
#include "diz.h"

extern hash aliases;
extern const char *empty;

static const char *ush_reserved_variable_names[] = { "r", "$", "n", "MAX_CMD_LENGHT", "HISTORY_SIZE", "MAX_VARIABLE_NAME_LENGHT", NULL };
#define is_reserved_variable( name ) stracmp( name, ush_reserved_variable_names )

void cd(const char *dir)
{
    if(!dir) return;
    if(!*dir) puts("ush: cd usage: cd <dir>");

    if ( chdir (dir) != 0 )
    {
        write_variable("r", "1");
        puts("ush: cd: No such file or directory");
    }
    else
        write_variable("r", "0");

    return;
}

void set(char **arg)
{
    char *name = arg[1];
    char *value = arg[2];

    #define set_help() puts("ush: = usage: set <variable name>[ <variable value>]\n")

    // checks variable name
    if( !name || !*name )
    {
        set_help();
        write_variable("r", "1");
        return;
    }
    else if( !isalpha( *name ) )
    {
        fprintf(stderr, "ush: %s: variable name must start with an alphabetic charater\n", name);
        write_variable("r", "1");
        return;
    }
    else if( strlen(name) >= atoi(read_variable("MAX_VARIABLE_NAME_LENGHT")) )
    {
        fprintf(stderr, "ush: %s: too long variable name\n", name);
        return;
    }
    else if( is_reserved_variable( name ) )
    {
        fprintf(stderr, "ush: %s: reserved variable name\n", name);
        return;
    }

    // checks variable value
    if( !value )
    {
        // if value is missing then replaces it with an empty string
        value = empty;
    }
    if( *value == '$' && strlen( value+sizeof(char)>0 ) )
    {
        // if value is a variable, expand it
        value = read_variable( value+sizeof(char) );
    }
    /*
    else if( *value != '\"' ||
             *(value+strlen(value)-1) != '\"' ) // value string must starts and ends with quotation marks
    {
        equal_help();
        return;
    }
    else
    {
        // cuts off quotation marks from value
        value += sizeof(char);
        *(value + strlen(value) - 1 ) = '\0';
    }
    */

    int rv = write_variable( name, value );
    write_variable("r", itoa(rv)); // saves return value

    #if USH_DEBUG
        printf("ush: debug: created new variable \"%s\"=\"%s\"\n", name, read_variable(name));
    #endif

    return;
}

void alias(char **args)
{
    char *name = args[1];
    char *value = calloc( sizeof(char), atoi(read_variable("MAX_CMD_LENGHT")) );
    int i = 2;
    while( args[i] )
    {
        strcat( value, args[i] );
        strcat( value, " " );
        i++;
    }

    #define alias_help() puts("ush: = usage: alias <alias name>[ <alias value>]\n")

    // checks alias name
    if( !name || !*name )
    {
        alias_help();
        write_variable("r", "1");
        return;
    }
    else if( !isalpha( *name ) )
    {
        fprintf(stderr, "ush: %s: alias must start with an alphabetic charater\n", name);
        write_variable("r", "1");
        return;
    }

    // checks alias value
    if ( !value )
    {
        // if value is missing then replaces it with an empty string
        value = empty;
    }

    // TODO: more checks!

    write_alias( name, value );

    free( value );
    return;
}
